<?php
/**
  * Common header for all m2f scripts.
  *
  * @author Patricio Anguita
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
  * @version $Revision: 1.93 $
  * @internal $Id: m2f_common.php,v 1.93 2007/03/09 13:29:20 georgeocrawford Exp $
*/

// This is used to make the version number that appears in the M2F settings page. The
// displayed version number is created by concatenating the m2f version with the db
// version with a dot in between. Increment the first two components for each
// major release (0.9, 1.0, 1.2, 2.0, etc.) and then increment the third component for
// each beta or bug fix release within the major release (1.0.0, 1.0.1, 1.0.2, etc.).
$m2f_version = '1.2.6';

// This version number identifies the DB schema that this script expects. Increment
// this number each time the DB schema is changed.
$m2f_desired_db_version = 12;

if (isset($_POST['m2f_root_path']) || isset($_GET['m2f_root_path']) || isset($_COOKIE['m2f_root_path'])) die ('Hacking attempt');

// Define main path, in case this file is called outside the root path (i.e. in admin/ directory)
if (!defined('M2F_ROOT_PATH')) 
{
	// pnphpbb specifics by ksg-serg
	if(isset($ModName)) 
	{
		$phpbb_root_path = './modules/' . $ModName . '/';
		define('M2F_ROOT_PATH', $phpbb_root_path.'m2f/');
	}
	else
	{
		define('M2F_ROOT_PATH', './');
	}
}

// Include install-specific mandatory configuration
if (file_exists(M2F_ROOT_PATH . 'm2f_config.php'))
{
  include_once (M2F_ROOT_PATH . 'm2f_config.php');
}
else
{
  m2f_display_error("m2f_config.php not found, check your installation!");
  die;
}


// Load proper values if phpBB DB is used
if (_M2F_USE_PHPBB_DB_)
{
  $type_db = $dbms;
	$user_db = $dbuser; 
	$pass_db = $dbpasswd; 
	$name_db = $dbname; 
	$host_db = $dbhost;
}

// phpBB uses mysql4 dbms when necessary, but PEAR makes no distinction in the db type
switch ($type_db) {
	case 'mysql4':
	case 'mysql':
	$type_db = 'mysql';
	require_once(M2F_ROOT_PATH."db/m2f_schema_mysql.php"); // include schema functions for this db
	break ;

  case 'postgres':
  $type_db = 'pgsql';
	require_once(M2F_ROOT_PATH."db/m2f_schema_pgsql.php"); // include schema functions for this db
  break;
}





// Set up PEAR on the path
if (_M2F_USE_LOCAL_PEAR_)
{
	$pear_root_path = M2F_ROOT_PATH . 'includes/pear/';

	if (!defined('PATH_SEPARATOR'))
	{
		$separator = strtoupper(substr(PHP_OS,0,3)=='WIN') ? ';' : ':';
		define('PATH_SEPARATOR', $separator);
	}
	
	$new_include_path = $pear_root_path . PATH_SEPARATOR . '../' . $pear_root_path . PATH_SEPARATOR . ini_get('include_path'); 
	if (function_exists('set_include_path')) 
	{ 
		set_include_path($new_include_path); 
	} 
	else 
	{ 
	  @ini_set('include_path', $new_include_path); 
	} 
}

// Load libraries
require_once('Log.php');		// PEAR Logging layer
require_once('DB.php');		// PEAR DB layer
require_once('Mail.php');	// PEAR MAIL layer
require_once('Mail/mime.php'); // PEAR Mail_mime layer

// Load compatibility functions, if needed
if (!function_exists('html_entity_decode'))
{
	require_once('Compat/Function/html_entity_decode.php');
}
if (!function_exists('is_a'))
{
	require_once('Compat/Function/is_a.php');
}

// Instatiate database reusable object
$m2f_db = DB::connect( "$type_db://$user_db:$pass_db@$host_db/$name_db" );

// Instantiate log object(s)

// FILE:
if ((_M2F_LOG_METHOD_ == 'FILE') || (_M2F_LOG_METHOD_ == 'BOTH')) 
{
	$file_log  = &Log::singleton('file', _M2F_LOG_FILE_ , '[M2F]', array('mode' => 0600, 'timeFormat' => '%X %x') , constant(_M2F_LOG_LEVEL_));
	if (!$file_log) 
	{
		m2f_display_error("[" . __FILE__ . " : " . __LINE__ . "] Could not create log in the file at " . _M2F_LOG_FILE_);
		die;
	}
}

// DATABASE:
if (!(DB::isError($m2f_db)) && ((_M2F_LOG_METHOD_ == 'DATABASE') || (_M2F_LOG_METHOD_ == 'BOTH'))) 
{
	$sql_log = &Log::singleton('sql', _M2F_LOG_TABLE_, '[M2F]', array('db' => $m2f_db, 'sequence' => _M2F_LOG_TABLE_), constant(_M2F_LOG_LEVEL_));
	if (!$sql_log) 
	{
		m2f_display_error("[" . __FILE__ . " : " . __LINE__ . "] Could not create database log");
		die;
	}
}

// Add them together
$m2f_log = &Log::singleton('composite');
if (isset($file_log)) $m2f_log->addChild($file_log);
if (isset($sql_log)) $m2f_log->addChild($sql_log);
// If there's been a database error, we can still write the error to the FILE log if it exists
if (DB::isError($m2f_db))
{
	$m2f_error_msg = 'Could not connect to the database: <br /><b>'.$m2f_db->getMessage() . '</b>';
	m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__ , __FILE__ );
	die;
}

m2f_log(PEAR_LOG_INFO , "M2F is starting" , __LINE__ , __FILE__ );

// Report on Log creation
m2f_log(PEAR_LOG_DEBUG , "Log object instantiated" , __LINE__ , __FILE__ );

// Report on database connection
m2f_log(PEAR_LOG_DEBUG , "Connected to database" , __LINE__ , __FILE__ );

// Set default DB fetch mode to associative
$m2f_db->setFetchMode(DB_FETCHMODE_ASSOC);

// Load configuration parameters
m2f_log(PEAR_LOG_DEBUG , "Attempting to load config parameters" , __LINE__ , __FILE__ );
m2f_get_config_parameters($m2f_db);

// Prune databases if necessary
m2f_prune_databases($m2f_db);

m2f_undo_add_slashes_phpBB();

// **********************************************************************************************
// *************************** GLOBAL FUNCTIONS START HERE **************************************
// **********************************************************************************************

// Global Logging function
function m2f_log ($log_level , $msg , $line , $file, $debug_var = '')
{
  global $m2f_log, $m2f_handler;


  if(!isset($m2f_log)) $m2f_log = &Log::singleton('composite');

  $file = basename($file);
  $dist = isset($m2f_handler->list_id) ? ", dist $m2f_handler->list_id" : '';
  if (is_null($m2f_handler->start_time)) $m2f_handler->start_time = array_sum(explode(" ",microtime()));
  $time = round((array_sum(explode(" ",microtime())) - $m2f_handler->start_time),2).'s';
  $log_msg = "[$file, line $line$dist, $time]\n$msg";
  
  if ($debug_var != '')
  {
    $log_msg .= " = " . m2f_debug_var($debug_var);
  }
  
  if ($log_level == PEAR_LOG_CRIT) 
  {
  	$log_msg_display = $log_msg . backtrace();
  	$log_msg = str_replace(array('<pre>', '<b>', '</b>', "</pre></div>\n"), '', $log_msg_display);
  	m2f_display_error($log_msg_display);
  }
  
  if (_M2F_LOG_METHOD_ == 'FILE')
	{
		$log_msg .= "\n";
	}
  
   $m2f_log->log($log_msg , $log_level);
}

// Global var debugging function
function m2f_debug_var($var)
{
  ob_start();
  print_r($var);
  $ret_str = ob_get_contents();
  ob_end_clean();
  
  return $ret_str;
}

// Check return value from DB call
function check_db_result($db_result, $sql, $line, $file)
{
  global $m2f_log;
  if (DB::isError($db_result))
  {
    if(!isset($m2f_log)) $m2f_log = &Log::singleton('composite');

    $m2f_error_msg = "DB error. sql: '$sql' error: " . $db_result->getMessage();
    m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , $line, $file);
    die;
  }
}

// For important errors which don't need to kill the script
function m2f_display_error($error)
{
	echo '<span class="code">[Mail2Forum] ' . $error . '</span><br />';
}


// Get config paramters
function m2f_get_config_parameters($m2f_db)
{
  //global $m2f_config , $m2f_db, $type_db,
  global $m2f_config , $type_db,
  $m2f_desired_db_version, $m2f_version;
  
  // See if M2F tables have been created, and read additional configuration directives from the database
  $sql = "SELECT config_name, config_value FROM " . _M2F_CONFIG_TABLE_;
  $db_result =& $m2f_db->getAssoc($sql, false, array(), DB_FETCHMODE_DEFAULT, true);
  
  if (DB::isError($db_result))
  {
    m2f_log(PEAR_LOG_WARNING , "M2F config table not found (first run?), attempting to build M2F tables" , __LINE__ , __FILE__ );

		// Try to build the DB schema
		require_once(M2F_ROOT_PATH.'db/m2f_schema.php');
		m2f_create_tables($type_db, $m2f_desired_db_version);
    m2f_log(PEAR_LOG_NOTICE , "M2F tables created" , __LINE__ , __FILE__ );
    
    // Try again
    $sql = "SELECT config_name, config_value FROM " . _M2F_CONFIG_TABLE_;
	  $db_result =& $m2f_db->getAssoc($sql, false, array(), DB_FETCHMODE_DEFAULT, true);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
  }
  
  // Check the M2F version and see if the tables need updating
  $cur_db_version = isset($db_result['_M2F_DB_VERSION_']) ? $db_result['_M2F_DB_VERSION_'][0] : 0;
  if ($cur_db_version < $m2f_desired_db_version)
  {
    include_once(M2F_ROOT_PATH."db/m2f_update_schema.php");

  	// Get updated configuration
    $sql = "SELECT config_name, config_value FROM " . _M2F_CONFIG_TABLE_;
	  $db_result =& $m2f_db->getAssoc($sql, false, array(), DB_FETCHMODE_DEFAULT, true);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
  }
  
  // Populate $m2f_config[] array
  $m2f_config = array();
  $m2f_config['_M2F_VERSION_'] = "$m2f_version.{$db_result['_M2F_DB_VERSION_'][0]}";
  m2f_log(PEAR_LOG_DEBUG , "Loading config values" , __LINE__ , __FILE__);
  foreach ($db_result as $key => $value)
  {
  	if (count($value) > 1)
  	{
  		$m2f_config[$key] = $value;
  	}
  	else
  	{
  		$m2f_config[$key] = $value[0];
  	}
  }

	// Try to set the uni-valued config directives as constants
	foreach ($m2f_config as $config_name => $config_value)
	{
		if (!defined($config_name) && !is_array($config_value))
		{
			define($config_name, $config_value);
		}
	}
  m2f_log(PEAR_LOG_DEBUG , "Config values: " , __LINE__ , __FILE__ , $m2f_config);
}


// Prune database tables if necessary:
// _M2F_LOG_TABLE_ and _M2F_CRON_TABLE_ obey the "_M2F_LOG_PRUNE_" setting
// _M2F_MESSAGE_ID_TABLE_ and _M2F_MESSAGE_HASH_TABLE_ obey the "_M2F_HASH_PRUNE_" setting
function m2f_prune_databases($m2f_db)
{
//	global $m2f_config, $m2f_db;
	global $m2f_config;
	
	if ($m2f_config['_M2F_LOG_PRUNE_'] > 0)
	{
		m2f_log(PEAR_LOG_DEBUG , "Starting automatic database log prune" , __LINE__ , __FILE__);

		// point at which record is out of date
		$log_prune = time() - ((int) $m2f_config['_M2F_LOG_PRUNE_'] * 86400);

		$sql = ($m2f_db->dbsyntax == 'mysql') ? "SELECT MIN(logtime) FROM " . _M2F_LOG_TABLE_ : (($m2f_db->dbsyntax == 'pgsql') ? "SELECT TO_CHAR (MIN(logtime),'YYYY-MM-DD HH24:MI:SS') FROM " . _M2F_LOG_TABLE_ : '');
		$oldest_log =& $m2f_db->getOne($sql);
		check_db_result($oldest_log, $sql, __LINE__, __FILE__);

		$log_prune_point = ($m2f_db->dbsyntax == 'mysql') ? date('YmdHis', $log_prune) : (($m2f_db->dbsyntax == 'pgsql') ? date('Y-m-d H:i:s', $log_prune) : '');
		if ($log_prune_point > $oldest_log)
		{
			$sql = "delete from " . _M2F_LOG_TABLE_ . " where logtime < '" . $log_prune_point . "'";
			$db_result =& $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
			m2f_log(PEAR_LOG_DEBUG , "Deleted " . $m2f_db->affectedRows() . ' old Log entries' , __LINE__ , __FILE__);
		}

		// point at which record is out of date
		$cron_prune = time() - ((int) $m2f_config['_M2F_LOG_PRUNE_'] * 86400); 
	
		$sql = "SELECT MIN(logtime) FROM " . _M2F_CRON_TABLE_;
		$oldest_cron =& $m2f_db->getOne($sql);
		check_db_result($oldest_cron, $sql, __LINE__, __FILE__);

		if ($oldest_cron && ($cron_prune > $oldest_cron))
		{
			$sql = "delete from " . _M2F_CRON_TABLE_ . " where logtime < '" . $cron_prune . "'";
			$db_result =& $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
			m2f_log(PEAR_LOG_DEBUG , "Deleted " . $m2f_db->affectedRows() . ' old Cron records' , __LINE__ , __FILE__);
		}
	}

	if ($m2f_config['_M2F_HASH_PRUNE_'] > 0)
	{
		m2f_log(PEAR_LOG_DEBUG , "Starting automatic message hash prune" , __LINE__ , __FILE__);

		// point at which record is out of date
		$hash_prune = time() - ((int) $m2f_config['_M2F_HASH_PRUNE_'] * 86400);
	
		$sql = "SELECT MIN(logtime) FROM " . _M2F_MESSAGE_ID_TABLE_;
		$oldest_id =& $m2f_db->getOne($sql);
		check_db_result($oldest_id, $sql, __LINE__, __FILE__);
	
		$sql = "SELECT MIN(logtime) FROM " . _M2F_MESSAGE_HASH_TABLE_;
		$oldest_hash =& $m2f_db->getOne($sql);
		check_db_result($oldest_hash, $sql, __LINE__, __FILE__);
		
		if ($oldest_id && ($hash_prune > $oldest_id))
		{
			$sql = "delete from " . _M2F_MESSAGE_ID_TABLE_ . " where logtime < '" . $hash_prune . "'";
			$db_result =& $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
			m2f_log(PEAR_LOG_DEBUG , "Deleted " . $m2f_db->affectedRows() . ' old Message ID records' , __LINE__ , __FILE__);
		}
	
		if ($oldest_hash && ($hash_prune > $oldest_hash))
		{
			$sql = "delete from " . _M2F_MESSAGE_HASH_TABLE_ . " where logtime < '" . $hash_prune . "'";
			$db_result =& $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
			m2f_log(PEAR_LOG_DEBUG , "Deleted " . $m2f_db->affectedRows() . ' old Message Hash records' , __LINE__ , __FILE__);
		}
	}
	
}

function backtrace()
{
   $output .= "\n\n<pre><b>Backtrace:</b>\n";
   $backtrace = debug_backtrace();
	 unset ($backtrace[0]);
	 
   foreach ($backtrace as $bt) {
       $args = '';
				 if (is_array($bt['args']))
				 foreach ($bt['args'] as $a) {
           if (!empty($args)) {
               $args .= ', ';
           }
           switch (gettype($a)) {
           case 'integer':
           case 'double':
               $args .= $a;
               break;
           case 'string':
               $a = htmlspecialchars(substr($a, 0, 120)).((strlen($a) > 120) ? '...' : '');
               $args .= "\"$a\"";
               break;
           case 'array':
               $args .= 'Array('.count($a).')';
               break;
           case 'object':
               $args .= 'Object('.get_class($a).')';
               break;
           case 'resource':
               $args .= 'Resource('.strstr($a, '#').')';
               break;
           case 'boolean':
               $args .= $a ? 'True' : 'False';
               break;
           case 'NULL':
               $args .= 'Null';
               break;
           default:
               $args .= 'Unknown';
           }
       }
       $output .= "\n";
       $output .= "<b>file:</b> {$bt['line']} - {$bt['file']}\n";
       $output .= "<b>call:</b> {$bt['class']}{$bt['type']}{$bt['function']}($args)\n";
   }
   $output .= "</pre></div>\n";
   return $output;
}





// Strip slashes which were automatically added - this is for phpBB compatibility, as all $_SERVER variables have had slashes added!
function m2f_undo_add_slashes_phpBB()
{
	if( get_magic_quotes_gpc() )
	{
		if( is_array($_GET) )
		{
			while( list($k, $v) = each($_GET) )
			{
				if( is_array($_GET[$k]) )
				{
					while( list($k2, $v2) = each($_GET[$k]) )
					{
						$_GET[$k][$k2] = stripslashes($v2);
					}
					@reset($_GET[$k]);
				}
				else
				{
					$_GET[$k] = stripslashes($v);
				}
			}
			@reset($_GET);
		}
	
		if( is_array($_POST) )
		{
			while( list($k, $v) = each($_POST) )
			{
				if( is_array($_POST[$k]) )
				{
					while( list($k2, $v2) = each($_POST[$k]) )
					{
						$_POST[$k][$k2] = stripslashes($v2);
					}
					@reset($_POST[$k]);
				}
				else
				{
					$_POST[$k] = stripslashes($v);
				}
			}
			@reset($_POST);
		}
	}
}

?>
